#ifndef	___ACTextControl_H_
#define	___ACTextControl_H_
//------------------------------------------------------------------------------
// BeOS
#include <ListView.h>
#include <Locker.h>
#include <MessageFilter.h>
#include <interface/StringView.h>
#include <TextControl.h>
// C++
#include <list>
#include <string>
// Add2
#include <add2/GUIBase.h>
#include <add2/ToolBar.h>
//------------------------------------------------------------------------------
const uint32	AC_TXT_CTRL_DISPLAY_MAX_CANDIDATES = 9;
//------------------------------------------------------------------------------
// LKKs
const char	LKK_ACC_VIEW_TBUTTON_SORT_ALPHA[] = "Alphabetical";
const char	LKK_ACC_VIEW_TBUTTON_SORT_LRU[] = "LRU";
//==============================================================================
class	TACCandidatesView : public TDialogView
{
public:
	TACCandidatesView(BRect oFrame, BHandler* oTgt);
	virtual	~TACCandidatesView();

	void	Cancel();				// 確定してもキャンセルしても呼び出す
	void	SetCandidates(const list<string>& oCandidates);

private:
	typedef	TDialogView	SUPER;

	virtual void	AllAttached();
	void			CandidateDetered(BMessage* oMsg);
	void			Deter();
	void			LViewCandidatesSelected();
	virtual void	MessageReceived(BMessage* oMsg);
	void			SetViewLabels();
	virtual void	ShortCutDown(int32 oChar, int32 oFunc);
	void			TButtonSortClicked();

	list<string>	mCandidates;
	list<string>	mDisplayCandidates;
	BHandler*		mTgt;
	BListView*		mLViewCandidates;
	TToolBar*		mToolBar;
	TToolButton*	mTButtonAlpha;
	TToolButton*	mTButtonLRU;
};
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
候補一覧の表示、キー・マウスによる選択処理など
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TACCandidatesWindow : public TDialogWindow
{
public:
	TACCandidatesWindow(BRect oFrame, BHandler* oTgt);
	virtual	~TACCandidatesWindow();

	void	SetCandidates(const list<string>& oCandidates);

private:
	typedef	TDialogWindow	SUPER;

	bool				QuitRequested();

	TACCandidatesView*	mCandidatesView;
};
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
単に最有力候補を表示するだけ
キー処理などは行わない
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TACFavoriteWindow : public TBasicWindow
{
public:
	TACFavoriteWindow(BRect oFrame);
	virtual	~TACFavoriteWindow();

	void	SetFavorite(const string& oFavorite);
	string	Favorite() const;

private:
	typedef	TBasicWindow	SUPER;

	string			mFavorite;
	BStringView*	mStrViewFavorite;
};
//==============================================================================
class	TACTCMessageFilter : public BMessageFilter
{
public:
	TACTCMessageFilter(BHandler* oTgt);
	virtual	~TACTCMessageFilter();

private:
	typedef	BMessageFilter	SUPER;

	virtual filter_result	Filter(BMessage* oMsg, BHandler** oTgt);

	BHandler*	mTgt;
};
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
カーソル位置までの文字列を対象に補完候補を選出する
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TACTextControl : public BTextControl
{
public:
	TACTextControl(BRect oFrame, const char* oName, const char* oLabel,
			const char* oText, BMessage* oMsg, uint32 oResizingMode = B_FOLLOW_LEFT|B_FOLLOW_TOP,
			uint32 oFlags = B_WILL_DRAW|B_NAVIGABLE);
	virtual	~TACTextControl();

	status_t	SetTo(const string& oPath);			// 候補を保存するファイル

	void		AddCandidate(const string& oCandidate);
//	void		RemoveCandidate(const string& oCandidate);

	status_t	SetMaxCandidates(uint32 oMax);

	void		SetTrigger(const string& oTrigger);

	void		Learn();

private:
	typedef	BTextControl	SUPER;

	void			AddCandidates(const BMessage& oArc);
	virtual void	AllAttached();
	virtual void	AllDetached();
	void			CandidateDetered(BMessage* oMsg);
	void			CandidateWindowClosed();
	void			Deter(const char* oCandidate);
	virtual void	DrawAfterChildren(BRect oUpdateRect);
	status_t		GetMatchCandidates(string* oFavorite, list<string>* oMatchCandidates) const;
	void			InsertCandidate(const string& oCandidate);
	bool			IsFavoriteWindowOpened() const;
	virtual void	KeyDown(const char* oBytes, int32 oNumBytes);
	void			ManageFavoriteWindow(bool oIsFavoriteOpen, bool oShouldFavoriteOpen, const string& oFavorite);
	void			MateCandidatesUpdated(BMessage* oMsg);
	virtual void	MessageReceived(BMessage* oMsg);
	status_t		Save() const;
	int32			SearchTargetEndPos() const;
//	bool			ShouldFavoriteWindowOpened(string* oFavorite) const;
	void			ShowCandidatesWindow();

	bool					mPrevFocused;
	uint32					mMaxCandidates;
	string					mLastAutoLearningText;
	string					mCandidatesStorageFileName;
	string					mTrigger;
	list<string>			mCandidates;
	TACCandidatesWindow*	mCandidatesWindow;
	TACFavoriteWindow*		mFavoriteWindow;
	TACTCMessageFilter*		mFilter;

	static BLocker	smInstancesLocker;
	static list<TACTextControl*>	smInstances;
};
//==============================================================================
#endif	// ___ACTextControl_H_
