/*==============================================================================
		ビットマップボタン		for ZETA		Ver 1.00
		Copyright (C) 2006 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2006/01/22 (Sun) 開発着手。
 1.00 | 2006/01/22 (Sun) ツールボタンクラスのコードを元に構築。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "BitmapButton.h"
//------------------------------------------------------------------------------
// BeOS
#include <Bitmap.h>
#include <Window.h>
// C++
// Add2
#include <add2/MiscDebug.h>
// Proj
//------------------------------------------------------------------------------
// float
const float		MARGIN_IMG_TEXT = 6.0;
const float		WIDTH_BORDER = 1.0;
//------------------------------------------------------------------------------
// rgb_color
const rgb_color	COLOR_BRIGHT_BORDER = { 255, 255, 255, 255 };
const rgb_color	COLOR_DARK_BORDER = { 128, 128, 128, 255 };
const rgb_color	COLOR_DISABLED_FILLED = { 128, 128, 128, 255 };
const rgb_color	COLOR_DISABLED_LABEL = { 128, 128, 128, 255 };
const rgb_color	COLOR_ENABLED_DISABLED_ON_BG = { 192, 192, 192, 255 };
const rgb_color	COLOR_ENABLED_LABEL = { 0, 0, 0, 255 };
const rgb_color	COLOR_HOT_OFF_BG = { 240, 240, 240, 255 };
//------------------------------------------------------------------------------
// BPoint
const BPoint	ON_OFFSET = BPoint(1, 1);
//------------------------------------------------------------------------------
//==============================================================================
TBitmapButton::TBitmapButton(BRect oFrame, const char* oName, BBitmap* oEnabledOff, BMessage* oMsg,
		uint32 oBehavior)
		: SUPER(oFrame, oName, "", oMsg, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	Init(oName, oEnabledOff, oBehavior);
}
//------------------------------------------------------------------------------
TBitmapButton::TBitmapButton(BRect oFrame, const char* oName, BBitmap* oEnabledOff, const char* oLabel, BMessage* oMsg,
		uint32 oBehavior)
		: SUPER(oFrame, oName, oLabel, oMsg, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	Init(oName, oEnabledOff, oBehavior);
}
//------------------------------------------------------------------------------
TBitmapButton::~TBitmapButton()
{
	for ( int32 i = 0 ; i < BB_STATE_END ; i++ ) {
		delete mImages[i];
	}
}
//------------------------------------------------------------------------------
void	TBitmapButton::AdjustDivider()
{
	if ( mImages[BB_STATE_ENABLED_OFF] != NULL ) {
		mDivider = mImages[BB_STATE_ENABLED_OFF]->Bounds().right+MARGIN_IMG_TEXT;
	}
}
//------------------------------------------------------------------------------
void	TBitmapButton::AttachedToWindow()
{
	SetDefaultImages();
}
//------------------------------------------------------------------------------
uint32	TBitmapButton::Behavior() const
{
	return mBehavior;
}
//------------------------------------------------------------------------------
void	TBitmapButton::CalcDrawPoints()
{
	font_height	aFontHeight;

	// 画像
	mImagePoint.x = WIDTH_BORDER;
	mImagePoint.y = WIDTH_BORDER;
	// ラベル
	GetFontHeight(&aFontHeight);
	mLabelPoint.x = mDivider;
	mLabelPoint.y = (Bounds().bottom-(aFontHeight.ascent+aFontHeight.descent))/2+aFontHeight.ascent;
	Invalidate();
}
//------------------------------------------------------------------------------
status_t	TBitmapButton::ClearImages()
{
	for ( int i = BB_STATE_ENABLED_ON ; i < BB_STATE_END ; i++ ) {
		delete mImages[i];
		mImages[i] = NULL;
	}
	return B_OK;
}
//------------------------------------------------------------------------------
void	TBitmapButton::ConvertFilledCopy(const BBitmap* oSrc,BBitmap** oDest,const rgb_color& oDestColor)
{
	rgb_color*	aDestBits;
	rgb_color*	aSrcBits;
	
	// Dest の作成
	*oDest = new BBitmap(oSrc->Bounds(),oSrc->ColorSpace());
	// 変換＆コピー
	aSrcBits = reinterpret_cast<rgb_color*>(oSrc->Bits());
	aDestBits = reinterpret_cast<rgb_color*>((*oDest)->Bits());
	for ( int32 i = 0 ; i < oSrc->BitsLength()/static_cast<int32>(sizeof(rgb_color)) ; i++ ) {
		if ( aSrcBits[i].alpha == 0 ) {	// 透明
			aDestBits[i] = aSrcBits[i];
		} else {	// 通常色
			aDestBits[i] = oDestColor;
		}
	}
}
//------------------------------------------------------------------------------
void	TBitmapButton::ConvertTransparentCopy(const BBitmap* oSrc,BBitmap** oDest,const rgb_color& oDestColor)
{
	rgb_color*	aDestBits;
	rgb_color*	aSrcBits;
	
	// Dest の作成
	*oDest = new BBitmap(oSrc->Bounds(),oSrc->ColorSpace());
	// 変換＆コピー
	aSrcBits = reinterpret_cast<rgb_color*>(oSrc->Bits());
	aDestBits = reinterpret_cast<rgb_color*>((*oDest)->Bits());
	for ( int32 i = 0 ; i < oSrc->BitsLength()/static_cast<int32>(sizeof(rgb_color)) ; i++ ) {
		if ( aSrcBits[i].alpha == 0 ) {	// 透明
			aDestBits[i].red = oDestColor.blue;
			aDestBits[i].green = oDestColor.green;
			aDestBits[i].blue = oDestColor.red;
			aDestBits[i].alpha = oDestColor.alpha;
		} else {	// 通常色
			aDestBits[i] = aSrcBits[i];
		}
	}
}
//------------------------------------------------------------------------------
BBitmap*	TBitmapButton::CreateSlidedBitmap(const BBitmap* oSrc, const rgb_color& oBGColor) const
{
	BBitmap*	aBmp = new BBitmap(oSrc->Bounds(), oSrc->ColorSpace(), true);
	BView*		aView = new BView(oSrc->Bounds(), "Canvas", B_FOLLOW_NONE, B_WILL_DRAW);

	aBmp->AddChild(aView);
	aBmp->Lock();
	aView->SetHighColor(oBGColor);
	aView->StrokeLine(BPoint(0, aView->Bounds().bottom), B_ORIGIN);
	aView->StrokeLine(BPoint(aView->Bounds().right, 0));
	aView->DrawBitmap(oSrc, ON_OFFSET);
	aView->Sync();
	aBmp->Unlock();
	return aBmp;
}
//------------------------------------------------------------------------------
void	TBitmapButton::Draw(BRect oUpdateRect)
{
	bool		aDrawFrame = false;
	int32		aImageIdx;
	float		aHeight, aWidth;
	rgb_color	aLeftTopColor, aRightBottomColor, aLabelColor, aBGColor;
	BPoint		aLabelOffset = B_ORIGIN;

	SUPER::Draw(oUpdateRect);
	// 表示画像の選択
		// Enabled, Disabled, Hot
	if ( !IsEnabled() ) {	// 無効状態
		aImageIdx = BB_STATE_DISABLED_OFF;
	} else if ( mHot ) {	// ホット
		aImageIdx = BB_STATE_HOT_OFF;
	} else {	// 有効で通常状態
		aImageIdx = BB_STATE_ENABLED_OFF;
	}
		// Off, On
	aImageIdx += ((Value() != B_CONTROL_OFF)|(mChanging&&mHot));
	if ( aImageIdx == BB_STATE_HOT_ON ) {
		aImageIdx = BB_STATE_ENABLED_ON;
	}
	// 描画プロパティ
	switch ( aImageIdx ) {
	case BB_STATE_ENABLED_OFF:
	case BB_STATE_DISABLED_OFF:
	case BB_STATE_HOT_OFF:
		aLeftTopColor = COLOR_BRIGHT_BORDER;
		aRightBottomColor = COLOR_DARK_BORDER;
		break;
	default:
		aLeftTopColor = COLOR_DARK_BORDER;
		aRightBottomColor = COLOR_BRIGHT_BORDER;
	}
	switch ( aImageIdx ) {
	case BB_STATE_ENABLED_OFF:
	case BB_STATE_DISABLED_OFF:
		aBGColor = ui_color(B_PANEL_BACKGROUND_COLOR);
		break;
	case BB_STATE_ENABLED_ON:
	case BB_STATE_DISABLED_ON:
		aBGColor = COLOR_ENABLED_DISABLED_ON_BG;
		aDrawFrame = true;
		aLabelOffset = ON_OFFSET;
		break;
	case BB_STATE_HOT_OFF:
		aBGColor = COLOR_HOT_OFF_BG;
		aDrawFrame = true;
		break;
	default:
		ASSERT(1,"TBitmapButton::Draw - bad ImgIdx");
	}
	if ( !mFlat ) {
		aDrawFrame = true;
	}
	switch (aImageIdx ) {
	case BB_STATE_DISABLED_OFF:
	case BB_STATE_DISABLED_ON:
		aLabelColor = COLOR_DISABLED_LABEL;
		break;
	default:
		aLabelColor = COLOR_ENABLED_LABEL;
	}
	aHeight = Bounds().bottom;
	aWidth = Bounds().right;
	// 背景の描画
	SetHighColor(aBGColor);
	FillRect(Bounds());
	// 画像の描画
	if ( mImages[aImageIdx] != NULL ) {
		DrawBitmap(mImages[aImageIdx], mImagePoint);
	}
	// ラベルの描画
//	DBEXP("TBitmapButton::Draw() - label", Label());
//	DBEXP("TBitmapButton::Draw() - label point", BRect(mLabelPoint, B_ORIGIN));
	SetHighColor(aLabelColor);
	DrawString(Label(), mLabelPoint+aLabelOffset);
	// 枠の描画
	if ( aDrawFrame ) {
		SetHighColor(aLeftTopColor);
		StrokeLine(BPoint(0, aHeight), BPoint(0,0));
		StrokeLine(BPoint(aWidth, 0));
		SetHighColor(aRightBottomColor);
		StrokeLine(BPoint(aWidth, aHeight));
		StrokeLine(BPoint(0, aHeight));
	}
}
//------------------------------------------------------------------------------
void	TBitmapButton::Init(const char* oName, BBitmap* oEnabledOff, uint32 oBehavior)
{
	// メンバへの引数からの代入
	mImages[BB_STATE_ENABLED_OFF] = oEnabledOff;
	// メンバ初期化
	mChanging = false;
	mFlat = false;
	mHot = false;
	mDivider = 0;
	mBehavior = oBehavior;
	mImageAlign = mLabelAlign = B_ALIGN_LEFT;
	for ( int i = BB_STATE_ENABLED_ON ; i < BB_STATE_END ; i++ ) {
		mImages[i] = NULL;
	}
	// ビュー設定
	SetViewColor(B_TRANSPARENT_COLOR);
	SetDrawingMode(B_OP_ALPHA);
	AdjustDivider();
	ResizeToPreferred();
}
//------------------------------------------------------------------------------
float	TBitmapButton::MinWidth() const
{
	float	aWidth = 0.0;

	if ( mImages[BB_STATE_ENABLED_OFF] != NULL ) {
		aWidth += mImages[BB_STATE_ENABLED_OFF]->Bounds().right+WIDTH_BORDER*2;
	}
	if ( Label()[0] != '\0' ) {
		aWidth += StringWidth(Label())+MARGIN_IMG_TEXT;
	}
	return aWidth;
}
//------------------------------------------------------------------------------
void	TBitmapButton::MouseDown(BPoint oPoint)
{
	SUPER::MouseDown(oPoint);
	if ( IsHidden() || !IsEnabled() )
		return;
	SetMouseEventMask(B_POINTER_EVENTS, B_NO_POINTER_HISTORY);
	mChanging = true;
	Invalidate();
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
oTransit の B_ENTERED_VIEW がバグっているので対処してある
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
void	TBitmapButton::MouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg)
{
	uint32	aGotButtons;
	BPoint	aGotPoint;

	SUPER::MouseMoved(oPoint, oTransit, oMsg);
	if ( IsHidden() )
		return;
	switch ( oTransit ) {
	case B_ENTERED_VIEW:
	case B_INSIDE_VIEW:
		GetMouse(&aGotPoint, &aGotButtons);
		if ( !mHot && (mChanging || Window()->IsActive() && aGotButtons == 0) ) {
			mHot = true;
			Invalidate();
		}
		break;
	case B_EXITED_VIEW:
//		DBEXP("TBitmapButton::MouseMoved","B_EXITED_VIEW");
		mHot = false;
		Invalidate();
		break;
	case B_OUTSIDE_VIEW:
		break;
	default:
		ASSERT(1, "TBitmapButton::MouseMoved - other");
	}
}
//------------------------------------------------------------------------------
void	TBitmapButton::MouseUp(BPoint oPoint)
{
	SUPER::MouseUp(oPoint);
	if ( IsHidden() )
		return;
//	DBEXP("TBitmapButton::MouseUp","");
	if ( !mChanging )
		return;
	// ボタン内で押されたマウスが放された
	mChanging = false;
	if ( !Bounds().Contains(oPoint) )	// ボタン外で放された
		return;
	// ボタン内で放された
	if ( Behavior() == B_TWO_STATE_BUTTON ) {
		SetValue(!Value());
		Invoke();
	} else {
		Invalidate();
		Invoke();
	}
}
//------------------------------------------------------------------------------
void	TBitmapButton::ResizeToPreferred()
{
	float		aImageHeight = 0.0;
	font_height	aFontHeight;

	if ( mImages[BB_STATE_ENABLED_OFF] != NULL ) {
		aImageHeight = mImages[BB_STATE_ENABLED_OFF]->Bounds().bottom;
	}
	GetFontHeight(&aFontHeight);
	ResizeTo(MinWidth(), max_c(aImageHeight, aFontHeight.ascent+aFontHeight.descent)+WIDTH_BORDER*2);
	CalcDrawPoints();
}
//------------------------------------------------------------------------------
void	TBitmapButton::SetAlignment(alignment oForImage, alignment oForLabel)
{
	mImageAlign = oForImage;
	mLabelAlign = oForLabel;
	CalcDrawPoints();
}
//------------------------------------------------------------------------------
status_t	TBitmapButton::SetBehavior(uint32 oBehavior)
{
	if ( oBehavior == Behavior() )
		return B_OK;
	// 変更
	mBehavior = oBehavior;
	if ( Behavior() == B_ONE_STATE_BUTTON ) {
		SetValue(B_CONTROL_OFF);	// 押されている状態かもしれないのであげる
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TBitmapButton::SetDefaultImages()
{
	rgb_color	aBGColor;
	rgb_color*	aSrcColor;

	if ( mImages[BB_STATE_ENABLED_OFF] == NULL )
		return B_ERROR;
	aSrcColor = reinterpret_cast<rgb_color*>(mImages[BB_STATE_ENABLED_OFF]->Bits());
	aBGColor.red = aSrcColor->blue;
	aBGColor.green = aSrcColor->green;
	aBGColor.blue = aSrcColor->red;
	aBGColor.alpha = aSrcColor->alpha;
	return SetDefaultImages(aBGColor);
}
//------------------------------------------------------------------------------
status_t	TBitmapButton::SetDefaultImages(const rgb_color& oBGColor)
{
	status_t	aResult = B_ERROR;
	BBitmap*	aSlidedBmp;

	if ( mImages[BB_STATE_ENABLED_OFF] == NULL )
		return aResult;
	// EnabledOff → DisabledOff, HotOff
	if ( mImages[BB_STATE_DISABLED_OFF] == NULL ) {
		ConvertFilledCopy(mImages[BB_STATE_ENABLED_OFF],&mImages[BB_STATE_DISABLED_OFF],COLOR_DISABLED_FILLED);
		aResult = B_OK;
	}
	if ( mImages[BB_STATE_HOT_OFF] == NULL ) {
		ConvertTransparentCopy(mImages[BB_STATE_ENABLED_OFF],&mImages[BB_STATE_HOT_OFF],COLOR_HOT_OFF_BG);
		aResult = B_OK;
	}
	// EnabledOff → EnabledOn
	if ( mImages[BB_STATE_ENABLED_ON] == NULL ) {
		aSlidedBmp = CreateSlidedBitmap(mImages[BB_STATE_ENABLED_OFF], oBGColor);
		ConvertTransparentCopy(aSlidedBmp,&mImages[BB_STATE_ENABLED_ON],COLOR_ENABLED_DISABLED_ON_BG);
		delete aSlidedBmp;
		aResult = B_OK;
	}
	// DisabledOff → DisabledOn
	if ( mImages[BB_STATE_DISABLED_ON] == NULL ) {
		aSlidedBmp = CreateSlidedBitmap(mImages[BB_STATE_DISABLED_OFF], oBGColor);
		ConvertTransparentCopy(mImages[BB_STATE_DISABLED_OFF],&mImages[BB_STATE_DISABLED_ON],COLOR_ENABLED_DISABLED_ON_BG);
		delete aSlidedBmp;
		aResult = B_OK;
	}
	return aResult;
}
//------------------------------------------------------------------------------
void	TBitmapButton::SetDivider(float oDivider)
{
	mDivider = oDivider;
	CalcDrawPoints();
}
//------------------------------------------------------------------------------
status_t	TBitmapButton::SetImage(int32 oWhich, BBitmap* oImg)
{
	if ( oWhich < 0 || oWhich >= BB_STATE_HOT_ON )
		return B_ERROR;
	if ( mImages[BB_STATE_ENABLED_OFF] == NULL && oWhich != BB_STATE_ENABLED_OFF )
		return B_ERROR;
	if ( oWhich != BB_STATE_ENABLED_OFF && mImages[BB_STATE_ENABLED_OFF]->Bounds() != oImg->Bounds() )
		return B_ERROR;
	delete mImages[oWhich];
	mImages[oWhich] = oImg;
	if ( oWhich == BB_STATE_ENABLED_OFF ) {
		AdjustDivider();
		ResizeToPreferred();
	}
	Invalidate();
	return B_OK;
}
//------------------------------------------------------------------------------
void	TBitmapButton::SetLabel(const char* oLabel)
{
	SUPER::SetLabel(oLabel);
	ResizeToPreferred();
}
//------------------------------------------------------------------------------
void	TBitmapButton::SetValue(int32 oValue)
{
	if ( oValue == Value() )
		return;
	SUPER::SetValue(oValue != B_CONTROL_OFF);
	Invalidate();
}
//------------------------------------------------------------------------------
void	TBitmapButton::WindowActivated(bool oActive)
{
	uint32	aButtons;
	BPoint	aPoint;

	if ( oActive ) {
		GetMouse(&aPoint,&aButtons);
		if ( Bounds().Contains(aPoint) ) {
			mHot = true;
			Invalidate();
		}
	} else {
		mHot = false;
		Invalidate();
	}
}
//------------------------------------------------------------------------------
//==============================================================================
