/*==============================================================================
		クリッカブル文字列ビュー	for ZETA	Ver 1.02
		Copyright (C) 2004-2005 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2004/09/19 (Sun) 開発着手。
 1.00 | 2004/09/19 (Sun) オリジナルバージョン。
==============================================================================*/

/*=== Memo =====================================================================
LView を自作しないとダメだ
	-> シングルクリックで選択できるように
	-> BS で候補ウィンドウを消せるように
終了時に落ちることがある。フィルタを delete しているからか？
==============================================================================*/

//------------------------------------------------------------------------------
#include "ClickableStringView.h"
//------------------------------------------------------------------------------
// BeOS
#include <Roster.h>
// C++
// Add2
#include <add2/MiscDebug.h>
#include <add2/MiscUtils.h>
//------------------------------------------------------------------------------
const float	MARGIN_UNDER_LINE = 1.0;
const float	PADDING = 2.0;
//==============================================================================
TClickableStringView::TClickableStringView(BRect oFrame, const char* oName, const char* oText,
		const char* oLinkToPath, const char* oLinkToMIME, BMessage* oMsg, uint32 oResizingMode, uint32 oFlags)
		: SUPER(oFrame, oName, oText, oMsg, oResizingMode, oFlags), TMouseDownManager(this)
{
	mLinkToPath = oLinkToPath;
	mLinkToMIME = oLinkToMIME;
	mFocusTgt = NULL;
}
//------------------------------------------------------------------------------
TClickableStringView::~TClickableStringView()
{
}
//------------------------------------------------------------------------------
void	TClickableStringView::AttachedToWindow()
{
	float	aHeight, aWidth;

	SUPER::AttachedToWindow();
	GetPreferredSize(&aWidth, &aHeight);
	ResizeTo(Bounds().right, aHeight);
}
//------------------------------------------------------------------------------
void	TClickableStringView::Draw(BRect oUpdateRect)
{
	DrawText();
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
テキストとアンダーラインの描画
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
void	TClickableStringView::DrawText()
{
	float		aBase;
	font_height	aFontHeight;
	rgb_color	aHighColorBak;

	GetFontHeight(&aFontHeight);
//	SetHighColor(0,0,0,255);
	DrawString(Label(), BPoint(PADDING, PADDING+aFontHeight.ascent));
	if ( !mLinkToPath.empty() ) {
		aHighColorBak = HighColor();
		DBEXP("TClickableStringView::DrawText() - isFocus", IsFocus());
		if ( IsFocus() ) {
			SetHighColor(ui_color(B_UI_NAVIGATION_BASE_COLOR));
		}
		aBase = PADDING+aFontHeight.ascent+aFontHeight.descent+MARGIN_UNDER_LINE;
		StrokeLine(BPoint(PADDING, aBase), BPoint(StringWidth(Label())+PADDING*2, aBase));
		SetHighColor(aHighColorBak);
	}
}
//------------------------------------------------------------------------------
BControl*	TClickableStringView::FocusTarget() const
{
	return mFocusTgt;
}
//------------------------------------------------------------------------------
void	TClickableStringView::GetPreferredSize(float* oWidth, float* oHeight)
{
	font_height	aFontHeight;

	*oWidth = StringWidth(Label())+PADDING*2;
	GetFontHeight(&aFontHeight);
	*oHeight = aFontHeight.ascent+aFontHeight.descent+MARGIN_UNDER_LINE+PADDING*2;
}
//------------------------------------------------------------------------------
const char*	TClickableStringView::LinkToMIME() const
{
	return mLinkToMIME.c_str();
}
//------------------------------------------------------------------------------
const char*	TClickableStringView::LinkToPath() const
{
	return mLinkToPath.c_str();
}
//------------------------------------------------------------------------------
void	TClickableStringView::MakeFocus(bool oFocused)
{
	SUPER::MakeFocus(oFocused);
	if ( oFocused && mFocusTgt != NULL ) {
		mFocusTgt->MakeFocus(true);
	}
}
//------------------------------------------------------------------------------
void	TClickableStringView::MouseClickLeftSingle(BPoint oPoint)
{
	char*		aArgV;

	if ( mLinkToPath.empty() )
		return;
	if ( mLinkToMIME.empty() ) {
		// MIME 指定無し
		show_help(mLinkToPath.c_str());
	} else {
		// MIME 指定有り
		aArgV = new char [mLinkToPath.length()+1];
		strcpy(aArgV, mLinkToPath.c_str());
		be_roster->Launch(mLinkToMIME.c_str(), 1, &aArgV);
		delete [] aArgV;
	}
}
//------------------------------------------------------------------------------
void	TClickableStringView::MouseDown(BPoint oPoint)
{
	SUPER::MouseDown(oPoint);
	ManageMouseDown(oPoint);
}
//------------------------------------------------------------------------------
void	TClickableStringView::MouseDownLeft(BPoint oPoint)
{
	rgb_color	aLowColor = LowColor();
	rgb_color	aHighColor = HighColor();

	if ( mLinkToPath.empty() )
		return;
	FillRect(Bounds());
	SetHighColor(aLowColor);
	SetLowColor(aHighColor);
	DrawText();
	SetHighColor(aHighColor);
	SetLowColor(aLowColor);
}
//------------------------------------------------------------------------------
void	TClickableStringView::MouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg)
{
	SUPER::MouseMoved(oPoint, oTransit, oMsg);
	ManageMouseMoved(oPoint, oTransit, oMsg);
}
//------------------------------------------------------------------------------
void	TClickableStringView::MouseUp(BPoint oPoint)
{
	SUPER::MouseUp(oPoint);
	ManageMouseUp(oPoint);
}
//------------------------------------------------------------------------------
void	TClickableStringView::MouseUpLeft(BPoint oPoint)
{
	Invalidate();
}
//------------------------------------------------------------------------------
void	TClickableStringView::SetFocusTarget(BControl* oTgt)
{
	mFocusTgt = oTgt;
}
//------------------------------------------------------------------------------
void	TClickableStringView::SetLinkToMIME(const char* oLinkToMIME)
{
	mLinkToMIME = oLinkToMIME;
}
//------------------------------------------------------------------------------
void	TClickableStringView::SetLinkToPath(const char* oLinkToPath)
{
	bool	aPrevEmpty = mLinkToPath.empty();

	mLinkToPath = oLinkToPath;
	if ( aPrevEmpty != mLinkToPath.empty() ) {
		Invalidate();
	}
}
//------------------------------------------------------------------------------
void	TClickableStringView::SetText(const char* oText)
{
	SetLabel(oText);
}
//------------------------------------------------------------------------------
const char*	TClickableStringView::Text() const
{
	return Label();
}
//------------------------------------------------------------------------------
//==============================================================================
