/*==============================================================================
		GUI 基本部品	for ZETA	Ver 1.44
		Copyright (C) 1999-2006 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/11/16 (Sun) 開発着手。
 1.00 | 2003/11/16 (Sun) add 時代のクラスを元に構築。
 1.10 | 2003/11/30 (Sun) フック関数に LanguageChanged() を追加。
 1.20 | 2003/12/07 (Sun) フック関数に IAPChanged() を追加。
 1.30 | 2004/02/07 (Sat) マウス関係を TMouseDownManager に任せた。
 1.40 | 2006/01/03 (Tue) マイナーバージョンアップの積み重ね。
==============================================================================*/

/*=== Memo =====================================================================
TSHIDlgWin は new しただけで Show() しないとデッドスレッドになる？
Win → View へ B_LANGUAGE_CHANGED を自動転送するようにしたので（2006/01/03）、二重転送しないように注意。
==============================================================================*/

//------------------------------------------------------------------------------
#include "GUIBase.h"
//------------------------------------------------------------------------------
// BeOS
#include <CheckBox.h>
#include <Control.h>
#include <Locale.h>
#include <MenuField.h>
#include <RadioButton.h>
#include <Screen.h>
// C++
// Add2
#include "ACTextControl.h"
#include "BeConstants.h"
#include "FUConstants.h"
#include "MiscDebug.h"
#include "MiscUtils.h"
#include "ShortCutFilter.h"
//==============================================================================
TBasicView::TBasicView(BRect oFrame, const char* oName,uint32 oResizingMode, uint32 oFlags)
		: SUPER(oFrame, oName, oResizingMode, oFlags), TMouseDownManager(this)
{
}
//------------------------------------------------------------------------------
TBasicView::TBasicView(BMessage* oArchive)
		: SUPER(oArchive), TMouseDownManager(this)
{
}
//------------------------------------------------------------------------------
TBasicView::~TBasicView()
{
}
//------------------------------------------------------------------------------
void	TBasicView::IAPChanged(BMessage* oMsg)
{
}
//------------------------------------------------------------------------------
void	TBasicView::LanguageChanged()
{
}
//------------------------------------------------------------------------------
void	TBasicView::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case B_LANGUAGE_CHANGED:
		LanguageChanged();
		break;
	case FUC_IAP_CHANGED:
		IAPChanged(oMsg);
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
void	TBasicView::MouseDown(BPoint oPoint)
{
	TMouseDownManager::ManageMouseDown(oPoint);
}
//------------------------------------------------------------------------------
void	TBasicView::MouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg)
{
	TMouseDownManager::ManageMouseMoved(oPoint, oTransit, oMsg);
}
//------------------------------------------------------------------------------
void	TBasicView::MouseUp(BPoint oPoint)
{
	TMouseDownManager::ManageMouseUp(oPoint);
}
//==============================================================================
TBasicWindow::TBasicWindow(BRect oFrame, const char* oTitle, window_type oType, uint32 oFlags, uint32 oWorkspace)
		: SUPER(oFrame, oTitle, oType, oFlags, oWorkspace)
{
	Init();
}
//------------------------------------------------------------------------------
TBasicWindow::TBasicWindow(BRect oFrame, const char* oTitle, window_look oLook, window_feel oFeel, uint32 oFlags, uint32 oWorkspace)
		: SUPER(oFrame, oTitle, oLook, oFeel, oFlags, oWorkspace)
{
	Init();
}
//------------------------------------------------------------------------------
TBasicWindow::~TBasicWindow()
{
}
//------------------------------------------------------------------------------
void	TBasicWindow::Appear()
{
	if ( IsHidden() ) {
		Show();
	}
	while ( !IsActive() ) {
		Activate();
	}
}
//------------------------------------------------------------------------------
void	TBasicWindow::IAPChanged(BMessage* oMsg)
{
}
//------------------------------------------------------------------------------
void	TBasicWindow::Init()
{
}
//------------------------------------------------------------------------------
BPoint	TBasicWindow::IntelligentMoveTo(BPoint oPoint)
{
	if ( oPoint.x+Bounds().right > BScreen().Frame().right-FUC_WIDTH_WIN_BORDER ) {
		oPoint.x = BScreen().Frame().right-Bounds().right-FUC_WIDTH_WIN_BORDER;
	}
	if ( oPoint.y+Bounds().bottom > BScreen().Frame().bottom-FUC_WIDTH_WIN_BORDER ) {
		oPoint.y = BScreen().Frame().bottom-Bounds().bottom-FUC_WIDTH_WIN_BORDER;
	}
	if ( oPoint.x < 0 ) {
		oPoint.x = 0;
	}
	if ( oPoint.y < 0 ) {
		oPoint.y = 0;
	}
	MoveTo(oPoint);
	return oPoint;
}
//------------------------------------------------------------------------------
BPoint	TBasicWindow::IntelligentCentering()
{
	BPoint	aPoint;

	aPoint.x = (BScreen().Frame().right-Bounds().right)/2;
	aPoint.y = (BScreen().Frame().bottom-Bounds().bottom)/2;
	MoveTo(aPoint);
	return aPoint;
}
//------------------------------------------------------------------------------
void	TBasicWindow::LanguageChanged()
{
}
//------------------------------------------------------------------------------
void	TBasicWindow::MessageReceived(BMessage* oMsg)
{
	switch( oMsg->what ) {
	case B_LANGUAGE_CHANGED:
		LanguageChanged();
		TransferToView(oMsg);
		break;
	case FUC_IAP_CHANGED:
		IAPChanged(oMsg);
		TransferToView(oMsg);
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
bool	TBasicWindow::QuitRequested()
{
	bool		aCanClose;	// 返値

	if ( CurrentMessage() == NULL ) {	// App から直接呼ばれた
		aCanClose = true;
	} else {	// クローズボタンが押された
		if ( !IsHidden() ) {
			Hide();
		}
		aCanClose = false;
	}
	return aCanClose;
}
//------------------------------------------------------------------------------
void	TBasicWindow::TransferToView(BMessage* oMsg)
{
	TBasicView*	aBasicView;

	aBasicView = dynamic_cast<TBasicView*>(ChildAt(0));
	if ( aBasicView != NULL ) {
		BMessenger(aBasicView).SendMessage(oMsg);
	}
}
//==============================================================================
TDialogView::TDialogView(BRect oFrame, const char* oName, uint32 oResizingMode, uint32 oFlags)
		: SUPER(oFrame, oName, oResizingMode, oFlags)
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}
//------------------------------------------------------------------------------
TDialogView::TDialogView(BMessage* oArchive)
		: SUPER(oArchive)
{
}
//------------------------------------------------------------------------------
TDialogView::~TDialogView()
{
}
//------------------------------------------------------------------------------
status_t	TDialogView::ApplyShortCut(const BView* oTgtView, int32 oChar, int32 oFunc) const
{
	status_t		aResult = B_ERROR;
	BButton*		aChildButton = NULL;
	BCheckBox*		aChildCBox = NULL;
	BControl*		aChildCtrl = NULL;
	BMenuField*		aChildMField = NULL;
	BRadioButton*	aChildRButton = NULL;
	BView*			aChild = NULL;
	TToolButton*	aChildTButton = NULL;

	for ( int i = 0 ; i < oTgtView->CountChildren() ; i++ ) {
		aChild = oTgtView->ChildAt(i);
		aChildCtrl = dynamic_cast<BControl*>(aChild);
		if ( aChildCtrl != NULL && aChildCtrl->IsEnabled() ) {
			if ( (aChildButton = dynamic_cast<BButton*>(aChild)) != NULL ) {
				// ボタン
				if ( short_cut_from_label(aChildButton->Label()) == oChar ) {
					click_button(aChildButton);
					aResult = B_OK;
					break;
				}
				if ( oChar == B_ESCAPE && string(aChildButton->Name()) == FUC_VIEW_NAME_BUTTON_CANCEL ) {
					click_button(aChildButton);
					aResult = B_OK;
					break;
				}
			} else if ( (aChildCBox = dynamic_cast<BCheckBox*>(aChild)) != NULL ) {
				// チェックボックス
				if ( short_cut_from_label(aChildCBox->Label()) == oChar ) {
					aChildCBox->MakeFocus();
					aChildCBox->SetValue(!aChildCBox->Value());
					aChildCBox->Invoke();
					aResult = B_OK;
					break;
				}
			} else if ( (aChildMField = dynamic_cast<BMenuField*>(aChild)) != NULL ) {
				// メニューフィールド
				if ( short_cut_from_label(aChildMField->Label()) == oChar ) {
					aChildMField->MakeFocus(true);
					aResult = B_OK;
					break;
				}
			} else if ( (aChildRButton = dynamic_cast<BRadioButton*>(aChild)) != NULL ) {
				// ラジオボタン
				if ( short_cut_from_label(aChildRButton->Label()) == oChar ) {
					aChildRButton->MakeFocus();
					aChildRButton->SetValue(B_CONTROL_ON);
					aChildRButton->Invoke();
					aResult = B_OK;
					break;
				}
			} else if ( (aChildTButton = dynamic_cast<TToolButton*>(aChild)) != NULL ) {
				// ツールボタン
				if ( short_cut_from_label(aChildRButton->Label()) == oChar ) {
					aChildTButton->SetValue(B_CONTROL_ON);
					aChildTButton->Invoke();
					aResult = B_OK;
					break;
				}
			} else if ( aChildCtrl != NULL ) {
				// コントロール一般
				if ( short_cut_from_label(aChildCtrl->Label()) == oChar ) {
					aChildCtrl->MakeFocus(true);
					aResult = B_OK;
					break;
				}
			}
		}
		if ( ApplyShortCut(aChild, oChar, oFunc) == B_OK ) {
			aResult = B_OK;
			break;
		}
	}
	return aResult;
}
//------------------------------------------------------------------------------
void	TDialogView::SetTargetToThis(BView* oParent, bool oForce) const
{
	BControl*	aControl;

	aControl = dynamic_cast<BControl*>(oParent);
#ifdef	DEBUGz
	if ( aControl != NULL && aControl->Name() == string("ButtonCancel") ) {
		DBEXP("TDialogView::SetTargetToThis() target", (const void*)aControl->Target());
		DBEXP("TDialogView::SetTargetToThis() target", aControl->Target()->Name());
	}
#endif
	if ( aControl != NULL && (aControl->Target() == NULL || aControl->Target() == aControl->Looper() || oForce) ) {
		aControl->SetTarget(this);
	}
	for ( int i = 0 ; i < oParent->CountChildren() ; i++ ) {
		SetTargetToThis(oParent->ChildAt(i), oForce);
	}
}
//------------------------------------------------------------------------------
void	TDialogView::ShortCutDown(int32 oChar, int32 oFunc)
{
	char	aByte;

//	DBEXP("TDialogView::ShortCutDown()", oChar);
	switch ( oChar ) {
	case B_ESCAPE:
		if ( Window()->CurrentFocus() != NULL
				&& dynamic_cast<TACTextControl*>(Window()->CurrentFocus()->Parent()) != NULL ) {
			aByte = oChar;
			Window()->CurrentFocus()->Parent()->KeyDown(&aByte, 1);
		} else {
			ApplyShortCut(this, oChar, oFunc);
		}
		break;
	case B_RETURN:
		if ( dynamic_cast<BButton*>(Window()->CurrentFocus()) != NULL ) {
			click_button(dynamic_cast<BButton*>(Window()->CurrentFocus()));
		} else if ( dynamic_cast<BTextView*>(Window()->CurrentFocus()) != NULL
				&& dynamic_cast<BTextControl*>(Window()->CurrentFocus()->Parent()) == NULL ) {
			aByte = oChar;
			Window()->CurrentFocus()->KeyDown(&aByte, 1);
		} else if ( Window()->CurrentFocus() != NULL
				&& dynamic_cast<TACTextControl*>(Window()->CurrentFocus()->Parent()) != NULL ) {
			aByte = oChar;
			Window()->CurrentFocus()->Parent()->KeyDown(&aByte, 1);
		} else {
			click_button(Window()->DefaultButton());
		}
		break;
	default:
		ApplyShortCut(this, oChar, oFunc);
		break;
	}
}
//------------------------------------------------------------------------------
//==============================================================================
TDialogWindow::TDialogWindow(BRect oFrame, const char* oTitle, window_type oType, uint32 oFlags, uint32 oWorkspace)
		: SUPER(oFrame, oTitle, oType, oFlags, oWorkspace)
{
	TShortCutFilter*	aFilter = new TShortCutFilter();

	AddCommonFilter(aFilter);
	mDefaultMsg = NULL;
}
//------------------------------------------------------------------------------
TDialogWindow::~TDialogWindow()
{
	delete mDefaultMsg;
}
//------------------------------------------------------------------------------
void	TDialogWindow::Hide()
{
	SUPER::Hide();
	set_thread_priority(Thread(), B_LOW_PRIORITY);
}
//------------------------------------------------------------------------------
status_t	TDialogWindow::Invoke(BMessage* oMsg)
{
	if ( oMsg == NULL ) {
		oMsg = mDefaultMsg;
	}
	return mTgtMsger.SendMessage(oMsg);
}
//------------------------------------------------------------------------------
void	TDialogWindow::KeyDown(BMessage* oMsg)
{
	int32			aChar;
	int32			aFuncKey;
	TDialogView*	aDlgView;

//	DBEXP("TDialogWindow::KeyDown()", "");
	aDlgView = dynamic_cast<TDialogView*>(ChildAt(0));
	if ( aDlgView == NULL )
		return;
	oMsg->FindInt32(B_MSG_NAME_RAW_CHAR, &aChar);
	oMsg->FindInt32(B_MSG_NAME_KEY, &aFuncKey);
	aDlgView->ShortCutDown(aChar, aFuncKey);
}
//------------------------------------------------------------------------------
BMessage*	TDialogWindow::Message() const
{
	return mDefaultMsg;
}
//------------------------------------------------------------------------------
void	TDialogWindow::MessageReceived(BMessage* oMsg)
{
	switch( oMsg->what ) {
	case B_KEY_DOWN:	// ショートカット
		KeyDown(oMsg);
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
status_t	TDialogWindow::SetMessage(BMessage* oMsg)
{
	delete mDefaultMsg;
	mDefaultMsg = oMsg;
	return B_OK;
}
//------------------------------------------------------------------------------
void	TDialogWindow::SetTarget(const BMessenger& oTgtMsger)
{
	mTgtMsger = oTgtMsger;
}
//------------------------------------------------------------------------------
void	TDialogWindow::Show()
{
	set_thread_priority(Thread(), B_DISPLAY_PRIORITY);
	SUPER::Show();
}
//------------------------------------------------------------------------------
BMessenger	TDialogWindow::Target() const
{
	return mTgtMsger;
}
//------------------------------------------------------------------------------
//==============================================================================
