#include "SimpleInput.h"

/*==============================================================================
		シンプル文字列入力ダイアログ for Zeta		Ver 1.10
		Copyright (C) 2000-2004 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/12/07 (Sun) 開発着手。
 1.00 | 2003/12/07 (Sun) add 時代のクラスを元に構築。
 1.10 | 2004/05/22 (Sat) ボタン幅を指定できるようにした。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
// BeOS
#include <Autolock.h>
// Add2
#include <add2/FUConstants.h>
#include <add2/MiscUtils.h>
//------------------------------------------------------------------------------
const float	MARGIN_VV = 10.0;
//==============================================================================
TSimpleInputView::TSimpleInputView(BRect oFrame, button_width oWidthStyle)
		: SUPER(oFrame, "SimpleInputView")
{
	// メンバ変数初期化
	switch ( oWidthStyle ) {
	case B_WIDTH_AS_USUAL:
	case B_WIDTH_FROM_LABEL:
	case B_WIDTH_FROM_WIDEST:
		mWidthStyle = oWidthStyle;
		break;
	default:
		mWidthStyle = B_WIDTH_AS_USUAL;
	}
	// ビューの作成（自動生成）
		mTxtCtrlText = new BTextControl(BRect(10, 10, 240, 29), "TxtCtrlData", "", "", NULL/*Msg*/);
		mTxtCtrlText->SetDivider(34);
	AddChild(mTxtCtrlText);
	// ボタンの作成
		mButtonCancel = new BButton(BRect(70, 40, 150, 40+FUC_HEIGHT_BUTTON_DEFAULT), "ButtonCancel", "", new BMessage(FUC_BUTTON_CANCEL_CLICKED));
	AddChild(mButtonCancel);
		mButtonOK = new BButton(BRect(160, 40, 240, 40+FUC_HEIGHT_BUTTON_DEFAULT), "ButtonDefault", "", new BMessage(FUC_BUTTON_OK_CLICKED));
	AddChild(mButtonOK);
	// ビューの設定
	ResizeTo(250, mButtonOK->Frame().bottom+MARGIN_VV);
	mTxtCtrlText->SetResizingMode(B_FOLLOW_LEFT_RIGHT);
	mButtonCancel->SetResizingMode(B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM);
	mButtonOK->SetResizingMode(B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM);
	mButtonOK->MakeDefault(true);
	// その他
}
//------------------------------------------------------------------------------
TSimpleInputView::~TSimpleInputView()
{
}
//------------------------------------------------------------------------------
void	TSimpleInputView::AdjustButtonArrange()
{
	mButtonOK->MoveTo(Bounds().right-(MARGIN_VV+mButtonOK->Bounds().right),
			mButtonOK->Frame().top);
	mButtonCancel->MoveTo(mButtonOK->Frame().left-(MARGIN_VV+mButtonCancel->Bounds().right),
			mButtonCancel->Frame().top);
}
//------------------------------------------------------------------------------
void	TSimpleInputView::AdjustButtonWidth(BButton* oButton)
{
	float	aWidth;

	switch ( mWidthStyle ) {
	case B_WIDTH_AS_USUAL:
		oButton->ResizeTo(FUC_WIDTH_BUTTON_DEFAULT, oButton->Bounds().bottom);
		break;
	case B_WIDTH_FROM_LABEL:
		adjust_button_width(oButton);
		break;
	case B_WIDTH_FROM_WIDEST:
		adjust_button_width(mButtonCancel);
		adjust_button_width(mButtonOK);
		aWidth = max_c(mButtonCancel->Bounds().right, mButtonOK->Bounds().right);
		mButtonCancel->ResizeTo(aWidth, mButtonCancel->Bounds().bottom);
		mButtonOK->ResizeTo(aWidth, mButtonOK->Bounds().bottom);
		break;
	}
}
//------------------------------------------------------------------------------
void	TSimpleInputView::AttachedToWindow()
{
	//ビューの設定（配置順）
	mTxtCtrlText->MakeFocus(true);
	mButtonCancel->SetTarget(this);
	mButtonOK->SetTarget(this);
}
//------------------------------------------------------------------------------
void	TSimpleInputView::ButtonCancelClicked()
{
	Window()->Hide();
	mTxtCtrlText->SetText(mPrevText.String());
}
//------------------------------------------------------------------------------
void	TSimpleInputView::ButtonOKClicked()
{
	type_code		aType;
	TDialogWindow*	aOwnerWin;

	aOwnerWin = dynamic_cast<TDialogWindow*>(Window());
	if ( aOwnerWin->Message()->GetInfo(SID_MSG_NAME_SIMPLE_INPUT_TEXT, &aType) == B_OK ) {	// 既にデータが追加されている
		aOwnerWin->Message()->RemoveName(SID_MSG_NAME_SIMPLE_INPUT_TEXT);
	}
	aOwnerWin->Message()->AddString(SID_MSG_NAME_SIMPLE_INPUT_TEXT, mTxtCtrlText->Text());
	aOwnerWin->Invoke();
	aOwnerWin->Hide();
	mPrevText = mTxtCtrlText->Text();
}
//------------------------------------------------------------------------------
void	TSimpleInputView::MessageReceived(BMessage* oMsg)
{
	switch ( oMsg->what ) {
	case FUC_BUTTON_CANCEL_CLICKED:
		ButtonCancelClicked();
		break;
	case FUC_BUTTON_OK_CLICKED:
		ButtonOKClicked();
		break;
	default:
		SUPER::MessageReceived(oMsg);
	}
}
//------------------------------------------------------------------------------
void	TSimpleInputView::SelectAll()
{
	mTxtCtrlText->MakeFocus(true);
	mTxtCtrlText->TextView()->SelectAll();
}
//------------------------------------------------------------------------------
void	TSimpleInputView::SetButtonLabel(file_panel_button oWhichButton, const char* oLabel)
{
	BButton*	aTgtButton;

	switch ( oWhichButton ) {
	case B_CANCEL_BUTTON:
		aTgtButton = mButtonCancel;
		break;
	case B_DEFAULT_BUTTON:
		aTgtButton = mButtonOK;
		break;
	default:
		return;
	}
	mButtonOK->MakeDefault(false);
	aTgtButton->SetLabel(oLabel);
	AdjustButtonWidth(aTgtButton);
	AdjustButtonArrange();
	mButtonOK->MakeDefault(true);
}
//------------------------------------------------------------------------------
void	TSimpleInputView::SetLabel(const char* oLabel)
{
	mTxtCtrlText->SetLabel(oLabel);
	adjust_txt_ctrl_divider(mTxtCtrlText);
}
//------------------------------------------------------------------------------
void	TSimpleInputView::SetText(const char* oText)
{
	mTxtCtrlText->SetText(oText);
}
//------------------------------------------------------------------------------
void	TSimpleInputView::ShortCutDown(int32 oChar, int32 oFunc)
{
	switch ( oChar ) {
	case B_ESCAPE:
		click_button(mButtonCancel);
		break;
	default:
		SUPER::ShortCutDown(oChar, oFunc);
	}
}
//------------------------------------------------------------------------------
const char*	TSimpleInputView::Text()
{
	return mTxtCtrlText->Text();
}
//------------------------------------------------------------------------------
//==============================================================================
TSimpleInputWindow::TSimpleInputWindow(BRect oFrame, const char* oTitle, button_width oWidthStyle)
		: SUPER(oFrame, oTitle)
{
	//主ビューの作成
	mSimpleInputView = new TSimpleInputView(oFrame.OffsetToCopy(B_ORIGIN), oWidthStyle);
	ResizeTo(mSimpleInputView->Frame().right, mSimpleInputView->Frame().bottom);
	AddChild(mSimpleInputView);
	//その他
	SetMessage(new BMessage(SID_SIMPLE_INPUT_TEXT_CHANGED));
}
//------------------------------------------------------------------------------
TSimpleInputWindow::~TSimpleInputWindow()
{
}
//------------------------------------------------------------------------------
void	TSimpleInputWindow::Appear()
{
	BAutolock	aLock(this);

	mSimpleInputView->SelectAll();
	SUPER::Appear();
}
//------------------------------------------------------------------------------
bool	TSimpleInputWindow::QuitRequested()
{
	mSimpleInputView->ButtonCancelClicked();
	return SUPER::QuitRequested();
}
//------------------------------------------------------------------------------
void	TSimpleInputWindow::SetButtonLabel(file_panel_button oWhichButton, const char* oLabel)
{
	BAutolock	aLock(this);

	if ( !aLock.IsLocked() )
		return;
	mSimpleInputView->SetButtonLabel(oWhichButton, oLabel);
}
//------------------------------------------------------------------------------
void	TSimpleInputWindow::SetLabel(const char* oLabel)
{
	BAutolock	aLock(this);

	if ( !aLock.IsLocked() )
		return;
	mSimpleInputView->SetLabel(oLabel);
}
//------------------------------------------------------------------------------
void	TSimpleInputWindow::SetText(const char* oText)
{
	BAutolock	aLock(this);

	if ( !aLock.IsLocked() )
		return;
	mSimpleInputView->SetText(oText);
}
//------------------------------------------------------------------------------
const char*	TSimpleInputWindow::Text()
{
	BAutolock	aLock(this);

	if ( !aLock.IsLocked() )
		return "";
	return mSimpleInputView->Text();
}
//------------------------------------------------------------------------------
//==============================================================================
