/*==============================================================================
		スプリッタコンポーネント		for Zeta		Ver 1.00
		Copyright (C) 2003-2004 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2004/01/28 (Wed) 開発着手。
 1.00 | 2004/01/28 (Wed) add 時代のクラスを元に構築。
==============================================================================*/

/*=== Memo =====================================================================
VU: TMouseDownManager を使う

自分が移動された場合のみ配下のサイズを変更する
自分の親がリサイズされた場合などは、配下はリサイズしない（でないと、複数の配下を登録できるシステムが破綻する）
==============================================================================*/

//------------------------------------------------------------------------------
#include "Splitter.h"
//------------------------------------------------------------------------------
// BeOS
// C++
// Add
// Proj
//------------------------------------------------------------------------------
const float		DEFAULT_MIN_SIZE = 20.0;
const rgb_color	COLOR_BRIGHT_EDGE = { 239, 239, 239, 255 };
const rgb_color	COLOR_DARK_EDGE = { 128, 128, 128, 255 };
//------------------------------------------------------------------------------
//==============================================================================
TSplitter::TSplitter(BRect oFrame, const char* oName, uint32 oResizingMode, orientation oPosture)
		: SUPER(oFrame, oName, oResizingMode, B_WILL_DRAW), mPosture(oPosture)
{
	// メンバ変数初期化
	mDragging = false;
	mMinSizeLT = mMinSizeRB = DEFAULT_MIN_SIZE;
	// ビュー設定
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}
//------------------------------------------------------------------------------
TSplitter::~TSplitter()
{
}
//------------------------------------------------------------------------------
void	TSplitter::AdjustAllFollowers()
{
	list<BView*>::iterator	p;

	p = mFollowerLT.begin();
	while ( p != mFollowerLT.end() ) {
		AdjustFollowerLeftTop(*p);
		p++;
	}
	p = mFollowerRB.begin();
	while ( p != mFollowerRB.end() ) {
		AdjustFollowerRightBottom(*p);
		p++;
	}
}
//------------------------------------------------------------------------------
void	TSplitter::AdjustFollowerLeftTop(BView* oFollower)
{
	if ( mPosture == B_VERTICAL ) {
		oFollower->ResizeTo(Frame().left-oFollower->Frame().left, oFollower->Bounds().bottom);
	} else {
		oFollower->ResizeTo(oFollower->Bounds().right, Frame().top-oFollower->Frame().top);
	}
	oFollower->Invalidate();
}
//------------------------------------------------------------------------------
void	TSplitter::AdjustFollowerRightBottom(BView* oFollower)
{
	float	aNewSize;

	if ( mPosture == B_VERTICAL ) {
		aNewSize = oFollower->Frame().right-Frame().right;
		oFollower->MoveTo(Frame().right, oFollower->Frame().top);
		oFollower->ResizeTo(aNewSize, oFollower->Bounds().bottom);
	} else {
		aNewSize = oFollower->Frame().bottom-Frame().bottom;
		oFollower->MoveTo(oFollower->Frame().left, Frame().bottom);
		oFollower->ResizeTo(oFollower->Bounds().right, aNewSize);
	}
	oFollower->Invalidate();
}
//------------------------------------------------------------------------------
void	TSplitter::AttachedToWindow()
{
}
//------------------------------------------------------------------------------
void	TSplitter::Draw(BRect oUpdateRect)
{
	if ( mPosture == B_VERTICAL ) {
		SetHighColor(COLOR_BRIGHT_EDGE);
		StrokeLine(B_ORIGIN, BPoint(0, Bounds().bottom));
		SetHighColor(COLOR_DARK_EDGE);
		StrokeLine(BPoint(Bounds().right-1, 0), BPoint(Bounds().right-1, Bounds().bottom));
//		SetHighColor(COLOR_DARK_EDGE_2);
//		StrokeLine(BPoint(Bounds().right-1, 0), BPoint(Bounds().right-1, Bounds().bottom));
	} else {
		SetHighColor(COLOR_BRIGHT_EDGE);
		StrokeLine(B_ORIGIN, BPoint(Bounds().right, 0));
		SetHighColor(COLOR_DARK_EDGE);
		StrokeLine(BPoint(0, Bounds().bottom-1), BPoint(Bounds().right, Bounds().bottom-1));
//		SetHighColor(COLOR_DARK_EDGE_2);
//		StrokeLine(BPoint(0, Bounds().bottom-1), BPoint(Bounds().right, Bounds().bottom-1));
	}
}
//------------------------------------------------------------------------------
void	TSplitter::MouseDown(BPoint oWhere)
{
	SetEventMask(B_POINTER_EVENTS,B_NO_POINTER_HISTORY);
	mDragging = true;
	mMouseDownPointOffset = oWhere;
}
//------------------------------------------------------------------------------
void	TSplitter::MouseMoved(BPoint oWhere, uint32 oCode, const BMessage* oMsg)
{
	if ( !mDragging )
		return;
	ConvertToParent(&oWhere);
	// チェック
	if ( mPosture == B_VERTICAL ) {
		if ( oWhere.x-mMouseDownPointOffset.x < mMinSizeLT
				|| Parent()->Bounds().right-(oWhere.x-mMouseDownPointOffset.x+Bounds().right) < mMinSizeRB )
			return;
	} else {
		if ( oWhere.y-mMouseDownPointOffset.y < mMinSizeLT
				|| Parent()->Bounds().bottom-(oWhere.y-mMouseDownPointOffset.y+Bounds().bottom) < mMinSizeRB )
			return;
	}
	// 自分を移動
	if ( mPosture == B_VERTICAL ) {
		MoveTo(oWhere.x-mMouseDownPointOffset.x, Frame().top);
	} else {
		MoveTo(Frame().left, oWhere.y-mMouseDownPointOffset.y);
	}
	// 配下をリサイズ
	AdjustAllFollowers();
}
//------------------------------------------------------------------------------
void	TSplitter::MouseUp(BPoint oWhere)
{
	SetEventMask(0);
	mDragging = false;
}
//------------------------------------------------------------------------------
status_t	TSplitter::RegisterFollowerLeftTop(BView* oFollower)
{
	if ( Parent() != oFollower->Parent() )
		return B_ERROR;
	// 配下にする
	mFollowerLT.push_back(oFollower);
	AdjustFollowerLeftTop(oFollower);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TSplitter::RegieterFollowerRightBottom(BView* oFollower)
{
	if ( Parent() != oFollower->Parent() )
		return B_ERROR;
	// 配下にする
	mFollowerRB.push_back(oFollower);
	AdjustFollowerRightBottom(oFollower);
	return B_OK;
}
//------------------------------------------------------------------------------
void	TSplitter::SetMinSizes(float oLeftTop, float oRightBottom)
{
	mMinSizeLT = oLeftTop;
	mMinSizeRB = oRightBottom;
}
//------------------------------------------------------------------------------
void	TSplitter::SetPosition(float oPos)
{
	if ( mPosture == B_VERTICAL ) {
		MoveTo(oPos, Frame().top);
	} else {
		MoveTo(Frame().top, oPos);
	}
	AdjustAllFollowers();
}
//------------------------------------------------------------------------------
//==============================================================================
