/*==============================================================================
		スレッドクラス	for ZETA	Ver 1.10
		Copyright (C) 1999-2006 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/11/16 (Sun) 開発着手。
 1.00 | 2003/11/16 (Sun) add 時代のクラスを元に構築。
 1.10 | 2006/02/19 (Sun) コピーコンストラクタを作った。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "Thread.h"
//------------------------------------------------------------------------------
// BeOS
// C++
// Add2
#include "MiscDebug.h"
//------------------------------------------------------------------------------
//==============================================================================
TThread::TThread(const char* oName, int32 oPriority)
{
	InitTThread(oName, oPriority);
}
//------------------------------------------------------------------------------
TThread::TThread(const TThread& oOrig)
{
	thread_info	aThreadInfo;

	get_thread_info(oOrig.ThreadID(), &aThreadInfo);
	InitTThread(aThreadInfo.name, aThreadInfo.priority);
}
//------------------------------------------------------------------------------
TThread::~TThread()
{
	if ( !mTerminated ) {
		kill_thread(mThreadID);
	}
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
SpawnThread() 終了後に呼ばれる。
SpawnThread() と同じスレッド（ = mThreadID ）が実行するようだ。
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
void	TThread::ExitThread(void* oCaller)
{
	TThread*	aCallerThread = reinterpret_cast<TThread*>(oCaller);

	aCallerThread->mTerminated = true;
	if ( aCallerThread->mAutoDelete ) {
		delete aCallerThread;
	}
}
//------------------------------------------------------------------------------
void	TThread::InitTThread(const char* oName, int32 oPriority)
{
	mAutoDelete = false;
	mTerminated = false;
	mTerminateRqtd = false;
	mThreadID = spawn_thread(SpawnThread, oName, oPriority, this);
}
//------------------------------------------------------------------------------
bool	TThread::IsSuspended() const
{
	thread_info	aThreadInfo;

	get_thread_info(mThreadID, &aThreadInfo);
	return (aThreadInfo.state == B_THREAD_SUSPENDED);
}
//------------------------------------------------------------------------------
bool	TThread::IsTerminated() const
{
	return mTerminated;
}
//------------------------------------------------------------------------------
status_t	TThread::Resume() const
{
	return resume_thread(mThreadID);
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
コンストラクタによって準備され、Resume() によって実行される
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
int32	TThread::SpawnThread(void* oCaller)
{
	int32		aResult;
	TThread*	aCallerThread;

	on_exit_thread(ExitThread, oCaller);
	aCallerThread = reinterpret_cast<TThread*>(oCaller);
	aResult = aCallerThread->Execute();
	return aResult;
}
//------------------------------------------------------------------------------
status_t	TThread::SetAutoDelete(bool oAutoDelete)
{
	mAutoDelete = oAutoDelete;
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TThread::SetName(const char* oName) const
{
	return rename_thread(mThreadID, oName);
}
//------------------------------------------------------------------------------
status_t	TThread::SetPriority(int32 oPriority) const
{
	return	set_thread_priority(mThreadID, oPriority);
}
//------------------------------------------------------------------------------
status_t	TThread::Suspend() const
{
	return suspend_thread(mThreadID);
}
//------------------------------------------------------------------------------
status_t	TThread::Terminate()
{
	mTerminateRqtd = true;
	return B_OK;
}
//------------------------------------------------------------------------------
thread_id	TThread::ThreadID() const
{
	return mThreadID;
}
//------------------------------------------------------------------------------
status_t	TThread::Wait(status_t* oExitValue) const
{
	status_t	aDummyExitValue;

	if ( oExitValue == NULL ) {
		oExitValue = &aDummyExitValue;
	}
	return wait_for_thread(mThreadID, oExitValue);
}
//------------------------------------------------------------------------------
//==============================================================================
