/*==============================================================================
		ツールバーコンポーネント		for ZETA		Ver 2.21
		Copyright (C) 2001-2006 by ＳＨＩＮＴＡ
==============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2003/12/07 (Sun) 開発着手。
 1.00 | 2003/12/07 (Sun) add 時代のクラスを元に構築。
 2.00 | 2003/12/24 (Wed) ラベルを付けられるようにした。
 2.10 | 2004/02/08 (Sun) ツールボタンは単独でも（ツールバーの子供でなくても）利用できるようにした。
 2.20 | 2004/09/18 (Sat) RelocateItems() を作った。
==============================================================================*/

/*=== Memo =====================================================================
【　To Do　】
TBitmapButton クラスを作成し、ツールボタンを TBitmapButton の派生にする。
TToolTip TToolButton::ToolTip() があったほうがよいかと。

【　仕様　】
セパレータで区切られるまでは、TToolButton は兄弟

【　実装　】
ツールチップが一旦表示されたら、他のボタンに行ってもすぐそのボタンのチップを表示
表示する直前に、ウィンドウのアクティブさとマウス位置がウィンドウの中かどうかチェック

OK ZETA で B_ENTERED_VIEW が送られてこない場合があるバグに対処
OK SetEventMask() を SetMouseEventMask() に。
==============================================================================*/

//------------------------------------------------------------------------------
#include "ToolBar.h"
//------------------------------------------------------------------------------
// BeOS
#include <Window.h>
// C++
#include <algorithm>
// Add2
#include "BeConstants.h"
#include "MiscDebug.h"
// Proj
//------------------------------------------------------------------------------
const float		MARGIN_TOOL_BAR = 3.0;
const float		MARGIN_TOOL_BUTTON = 3.0;
const float		TB_SEPARATOR_WIDTH = 8.0;
const rgb_color	COLOR_BRIGHT_BORDER = { 255, 255, 255, 255 };
const rgb_color	COLOR_DARK_BORDER = { 128, 128, 128, 255 };
const rgb_color	COLOR_DISABLED_FILLED = { 128, 128, 128, 255 };
const rgb_color	COLOR_DISABLED_LABEL = { 128, 128, 128, 255 };
const rgb_color	COLOR_ENABLED_DISABLED_ON_BG = { 192, 192, 192, 255 };
const rgb_color	COLOR_ENABLED_LABEL = { 0, 0, 0, 255 };
const rgb_color	COLOR_HOT_OFF_BG = { 240, 240, 240, 255 };
//------------------------------------------------------------------------------
//==============================================================================
TToolBar::TToolBar(BRect oFrame,const char* oName,uint32 oResizingMode)
		: SUPER(oFrame,oName,oResizingMode,B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE)
{
	// メンバ初期化
	mEnabled = true;
	mFlat = false;
	mMaxItemHeight = 0.0;
	// ビュー設定
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}
//------------------------------------------------------------------------------
TToolBar::~TToolBar()
{
}
//------------------------------------------------------------------------------
bool	TToolBar::AddItem(TToolButton* oItem)
{
	return AddItem(oItem,mItems.size());
}
//------------------------------------------------------------------------------
bool	TToolBar::AddItem(TToolButton* oItem,int32 oIndex)
{
	if ( oIndex < 0 || oIndex > static_cast<int32>(mItems.size()) )
		return false;
	// 自身の高さを調整
	if ( oItem->Bounds().bottom > mMaxItemHeight ) {
		mMaxItemHeight = oItem->Bounds().bottom;
		ResizeTo(Bounds().right,mMaxItemHeight+MARGIN_TOOL_BAR*2);
	}
	// アイテム挿入
	mItems.insert(mItems.begin()+oIndex,oItem);
	AddChild(oItem);
	oItem->mOwner = this;
	RelocateItems(oIndex);
	return true;
}
//------------------------------------------------------------------------------
bool	TToolBar::AddSeparatorItem()
{
	return AddItem(new TSeparatorButton(mMaxItemHeight));
}
//------------------------------------------------------------------------------
void	TToolBar::AttachedToWindow()
{
	// ウィンドウいっぱいに幅を広げる
	ResizeTo(Window()->Bounds().right,Bounds().bottom);
}
//------------------------------------------------------------------------------
int32	TToolBar::CountItems() const
{
	return mItems.size();
}
//------------------------------------------------------------------------------
void	TToolBar::Draw(BRect oUpdateRect)
{
	float	aHeight = Bounds().bottom;
	float	aWidth = Bounds().right;

	SetHighColor(COLOR_BRIGHT_BORDER);
	StrokeLine(BPoint(0,aHeight),BPoint(0,0));
	StrokeLine(BPoint(aWidth,0));
	SetHighColor(COLOR_DARK_BORDER);
	StrokeLine(BPoint(aWidth,aHeight));
	StrokeLine(BPoint(0,aHeight));
}
//------------------------------------------------------------------------------
TToolButton*	TToolBar::FindItem(const char* oName) const
{
	int32			i = 0;
	TToolButton*	aTButton;

	while ( (aTButton = ItemAt(i)) != NULL ) {
		if ( string(oName) == aTButton->Name() )
			break;
		i++;
	}
	return aTButton;
}
//------------------------------------------------------------------------------
TToolButton*	TToolBar::FindPushed(int32 oBegin) const
{
	int32			i = oBegin;
	TToolButton*	aTButton;

	while ( (aTButton = ItemAt(i)) != NULL ) {
		if ( aTButton->Value() == B_CONTROL_ON )
			break;
		i++;
	}
	return aTButton;
}
//------------------------------------------------------------------------------
void	TToolBar::GetPreferredSize(float* oWidth,float* oHeight)
{
	// 幅
	if ( CountItems() == 0 ) {
		*oWidth = MARGIN_TOOL_BAR;
	} else {
		*oWidth = ItemAt(CountItems()-1)->Frame().right+MARGIN_TOOL_BAR;
	}
	// 高さ
	*oHeight = mMaxItemHeight+MARGIN_TOOL_BAR*2;
}
//------------------------------------------------------------------------------
int32	TToolBar::IndexOf(TToolButton* oItem) const
{
	vector<TToolButton*>::const_iterator	p;

	p = find(mItems.begin(),mItems.end(),oItem);
	if ( p == mItems.end() )
		return -1;
	else
		return p-mItems.begin();
}
//------------------------------------------------------------------------------
bool	TToolBar::IsEnabled() const
{
	return mEnabled;
}
//------------------------------------------------------------------------------
bool	TToolBar::IsFlat() const
{
	return mFlat;
}
//------------------------------------------------------------------------------
TToolButton*	TToolBar::ItemAt(int32 oIndex) const
{
	if ( oIndex < 0 || oIndex >= static_cast<int32>(mItems.size()) )
		return NULL;
	return mItems[oIndex];
}
//------------------------------------------------------------------------------
void	TToolBar::RelocateItems(int32 oIndex)
{
	float	aLeft;
	vector<TToolButton*>::iterator	p;

	if ( oIndex < 0 || oIndex >= CountItems() )
		return;
	// 配置
	if ( oIndex == 0 ) {
		aLeft = MARGIN_TOOL_BAR;
	} else {
		aLeft = mItems[oIndex-1]->Frame().right+1;
	}
	p = mItems.begin()+oIndex;
	while ( p != mItems.end() ) {
		(*p)->MoveTo(aLeft,MARGIN_TOOL_BAR);
		aLeft = (*p)->Frame().right+1;
		p++;
	}
}
//------------------------------------------------------------------------------
void	TToolBar::SetEnabled(bool oEnabled)
{
	mEnabled = oEnabled;
}
//------------------------------------------------------------------------------
status_t	TToolBar::SetFlat(bool oFlat)
{
	int32			i = 0;
	TToolButton*	aTButton;

	mFlat = oFlat;
	while ( (aTButton = ItemAt(i)) != NULL ) {
		aTButton->Invalidate();
		i++;
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TToolBar::SetTargetForItems(BHandler* oHandler)
{
	return SetTargetForItems(BMessenger(oHandler));
}
//------------------------------------------------------------------------------
status_t	TToolBar::SetTargetForItems(BMessenger oMsger)
{
	vector<TToolButton*>::iterator	p;

	p = mItems.begin();
	while ( p != mItems.end() ) {
		(*p)->SetTarget(oMsger);
		p++;
	}
	return B_OK;
}
//------------------------------------------------------------------------------
//==============================================================================
TToolButton::TToolButton(BRect oFrame, const char* oName, BBitmap* oEnabledOff, BMessage* oMsg,
		uint32 oBehavior)
		: SUPER(oFrame, oName, "", oMsg, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	Init(oName, oEnabledOff, oBehavior);
}
//------------------------------------------------------------------------------
TToolButton::TToolButton(BRect oFrame, const char* oName, BBitmap* oEnabledOff, const char* oLabel, BMessage* oMsg,
		uint32 oBehavior)
		: SUPER(oFrame, oName, oLabel, oMsg, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	Init(oName, oEnabledOff, oBehavior);
}
//------------------------------------------------------------------------------
TToolButton::~TToolButton()
{
	delete mToolTip;
}
//------------------------------------------------------------------------------
void	TToolButton::AdjustDivider()
{
	if ( mImages[TB_STATE_ENABLED_OFF] != NULL ) {
		mDivider = mImages[TB_STATE_ENABLED_OFF]->Bounds().right+MARGIN_TOOL_BUTTON*2;
	}
}
//------------------------------------------------------------------------------
void	TToolButton::AttachedToWindow()
{
	mToolTip->SetTarget(this);
	SetDefaultImages();
}
//------------------------------------------------------------------------------
uint32	TToolButton::Behavior() const
{
	return mBehavior;
}
//------------------------------------------------------------------------------
void	TToolButton::CalcDrawPoints()
{
	font_height	aFontHeight;

	// 画像
	mImagePoint.x = MARGIN_TOOL_BUTTON;
	mImagePoint.y = MARGIN_TOOL_BUTTON;
	// ラベル
	GetFontHeight(&aFontHeight);
	mLabelPoint.x = mDivider+MARGIN_TOOL_BUTTON;
	mLabelPoint.y = (Frame().bottom-(aFontHeight.ascent+aFontHeight.descent))/2+aFontHeight.ascent;
	Invalidate();
}
//------------------------------------------------------------------------------
status_t	TToolButton::ClearImages()
{
	for ( int i = TB_STATE_ENABLED_ON ; i < TB_STATE_END ; i++ ) {
		delete mImages[i];
		mImages[i] = NULL;
	}
	return B_OK;
}
//------------------------------------------------------------------------------
void	TToolButton::ConvertFilledCopy(const BBitmap* oSrc,BBitmap** oDest,const rgb_color& oDestColor)
{
	rgb_color*	aDestBits;
	rgb_color*	aSrcBits;
	
	// Dest の作成
	*oDest = new BBitmap(oSrc->Bounds(),oSrc->ColorSpace());
	// 変換＆コピー
	aSrcBits = reinterpret_cast<rgb_color*>(oSrc->Bits());
	aDestBits = reinterpret_cast<rgb_color*>((*oDest)->Bits());
	for ( int32 i = 0 ; i < oSrc->BitsLength()/static_cast<int32>(sizeof(rgb_color)) ; i++ ) {
		if ( aSrcBits[i].alpha == 0 ) {	// 透明
			aDestBits[i] = aSrcBits[i];
		} else {	// 通常色
			aDestBits[i] = oDestColor;
		}
	}
}
//------------------------------------------------------------------------------
void	TToolButton::ConvertTransparentCopy(const BBitmap* oSrc,BBitmap** oDest,const rgb_color& oDestColor)
{
	rgb_color*	aDestBits;
	rgb_color*	aSrcBits;
	
	// Dest の作成
	*oDest = new BBitmap(oSrc->Bounds(),oSrc->ColorSpace());
	// 変換＆コピー
	aSrcBits = reinterpret_cast<rgb_color*>(oSrc->Bits());
	aDestBits = reinterpret_cast<rgb_color*>((*oDest)->Bits());
	for ( int32 i = 0 ; i < oSrc->BitsLength()/static_cast<int32>(sizeof(rgb_color)) ; i++ ) {
		if ( aSrcBits[i].alpha == 0 ) {	// 透明
			aDestBits[i].red = oDestColor.blue;
			aDestBits[i].green = oDestColor.green;
			aDestBits[i].blue = oDestColor.red;
			aDestBits[i].alpha = oDestColor.alpha;
		} else {	// 通常色
			aDestBits[i] = aSrcBits[i];
#if 0
			aDestBits[i].red = aSrcBits[i].blue;
			aDestBits[i].green = aSrcBits[i].green;
			aDestBits[i].blue = aSrcBits[i].red;
			aDestBits[i].alpha = aSrcBits[i].alpha;
#endif
		}
	}
}
//------------------------------------------------------------------------------
void	TToolButton::Draw(BRect oUpdateRect)
{
	bool		aDrawFrame = false;
	int32		aImageIdx;
	float		aHeight, aWidth;
	rgb_color	aLeftTopColor, aRightBottomColor, aLabelColor, aBGColor;
	BPoint		aOffsetPoint(B_ORIGIN);

	SUPER::Draw(oUpdateRect);
	// 表示画像の選択
		// Enabled, Disabled, Hot
	if ( !IsEnabled() || mOwner != NULL && !mOwner->IsEnabled() ) {	// 無効状態
		aImageIdx = TB_STATE_DISABLED_OFF;
	} else if ( mHot ) {	// ホット
		aImageIdx = TB_STATE_HOT_OFF;
	} else {	// 有効で通常状態
		aImageIdx = TB_STATE_ENABLED_OFF;
	}
		// Off, On
	aImageIdx += ((Value() != B_CONTROL_OFF)|(mChanging&&mHot));
	if ( aImageIdx == TB_STATE_HOT_ON ) {
		aImageIdx = TB_STATE_ENABLED_ON;
	}
	// 描画プロパティ
	switch ( aImageIdx ) {
	case TB_STATE_ENABLED_OFF:
	case TB_STATE_DISABLED_OFF:
	case TB_STATE_HOT_OFF:
		aLeftTopColor = COLOR_BRIGHT_BORDER;
		aRightBottomColor = COLOR_DARK_BORDER;
		break;
	default:
		aLeftTopColor = COLOR_DARK_BORDER;
		aRightBottomColor = COLOR_BRIGHT_BORDER;
		aOffsetPoint = BPoint(1, 1);
	}
	switch ( aImageIdx ) {
	case TB_STATE_ENABLED_OFF:
	case TB_STATE_DISABLED_OFF:
		aBGColor = ui_color(B_PANEL_BACKGROUND_COLOR);
		break;
	case TB_STATE_ENABLED_ON:
	case TB_STATE_DISABLED_ON:
		aBGColor = COLOR_ENABLED_DISABLED_ON_BG;
		aDrawFrame = true;
		break;
	case TB_STATE_HOT_OFF:
		aBGColor = COLOR_HOT_OFF_BG;
		aDrawFrame = true;
		break;
	default:
		ASSERT(1,"TToolButton::Draw - bad ImgIdx");
	}
	if ( mOwner == NULL || !mOwner->IsFlat() ) {
		aDrawFrame = true;
	}
	switch (aImageIdx ) {
	case TB_STATE_DISABLED_OFF:
	case TB_STATE_DISABLED_ON:
		aLabelColor = COLOR_DISABLED_LABEL;
		break;
	default:
		aLabelColor = COLOR_ENABLED_LABEL;
	}
	aHeight = Bounds().bottom;
	aWidth = Bounds().right;
	// 背景の描画
	SetHighColor(aBGColor);
	FillRect(Bounds());
	// 枠の描画
	if ( aDrawFrame ) {
		SetHighColor(aLeftTopColor);
		StrokeLine(BPoint(0, aHeight), BPoint(0,0));
		StrokeLine(BPoint(aWidth, 0));
		SetHighColor(aRightBottomColor);
		StrokeLine(BPoint(aWidth, aHeight));
		StrokeLine(BPoint(0, aHeight));
	}
	// 画像の描画
	if ( mImages[aImageIdx] != NULL ) {
		DrawBitmap(mImages[aImageIdx], mImagePoint+aOffsetPoint);
	}
	// ラベルの描画
	SetHighColor(aLabelColor);
	DrawString(Label(), mLabelPoint+aOffsetPoint);
}
//------------------------------------------------------------------------------
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
この関数は TToolBar に移してもいいかも
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
void	TToolButton::GetBrothers(vector<TToolButton*>* oBrothers)
{
	int32	aSelfIdx,aIdx;

	if ( mOwner == NULL )
		return;
	aSelfIdx = mOwner->IndexOf(this);
	// 前方検索
	aIdx = aSelfIdx-1;
	while ( aIdx >= 0 ) {
		if ( dynamic_cast<TSeparatorButton*>(mOwner->ItemAt(aIdx)) != NULL )
			break;
		oBrothers->push_back(mOwner->ItemAt(aIdx));
		aIdx--;
	}
	// 後方検索
	aIdx = aSelfIdx+1;
	while ( aIdx < mOwner->CountItems() ) {
		if ( dynamic_cast<TSeparatorButton*>(mOwner->ItemAt(aIdx)) != NULL )
			break;
		oBrothers->push_back(mOwner->ItemAt(aIdx));
		aIdx++;
	}
}
//------------------------------------------------------------------------------
void	TToolButton::Init(const char* oName, BBitmap* oEnabledOff, uint32 oBehavior)
{
	// メンバへの引数からの代入
	mImages[TB_STATE_ENABLED_OFF] = oEnabledOff;
	// メンバ初期化
	mAlreadyEntered = false;
	mChanging = false;
	mHot = false;
	mRadioMode = false;
	mDivider = 0;
	mBehavior = oBehavior;
	mImageAlign = mLabelAlign = B_ALIGN_LEFT;
	for ( int i = TB_STATE_ENABLED_ON ; i < TB_STATE_END ; i++ ) {
		mImages[i] = NULL;
	}
	mOwner = NULL;
	// ビュー設定
	SetViewColor(B_TRANSPARENT_COLOR);
	SetDrawingMode(B_OP_ALPHA);
	AdjustDivider();
	ResizeToPreferred();
	// ツールチップ
	mToolTip = new TToolTip(oName);
}
//------------------------------------------------------------------------------
bool	TToolButton::IsRadioMode() const
{
	return mRadioMode;
}
//------------------------------------------------------------------------------
float	TToolButton::MinWidth() const
{
	float	aWidth = 0.0;

	if ( mImages[TB_STATE_ENABLED_OFF] != NULL ) {
		aWidth += mImages[TB_STATE_ENABLED_OFF]->Bounds().right+MARGIN_TOOL_BUTTON*2;
	}
	if ( Label()[0] != '\0' ) {
		aWidth += StringWidth(Label())+MARGIN_TOOL_BUTTON*2;
	}
	return aWidth;
}
//------------------------------------------------------------------------------
void	TToolButton::MouseDown(BPoint oPoint)
{
	if ( IsHidden() || !IsEnabled() )
		return;
	SetMouseEventMask(B_POINTER_EVENTS,B_NO_POINTER_HISTORY);
	mChanging = true;
	Invalidate();
}
//------------------------------------------------------------------------------
void	TToolButton::MouseMoved(BPoint oPoint, uint32 oTransit, const BMessage* oMsg)
{
	uint32	aGotButtons;
	BPoint	aGotPoint;

	if ( IsHidden() )
		return;
	switch ( oTransit ) {
	case B_ENTERED_VIEW:
	case B_INSIDE_VIEW:
//		DBEXP("TToolButton::MouseMoved - B_ENTERED_VIEW", (const void*)oMsg);
		if ( !mAlreadyEntered ) {
			GetMouse(&aGotPoint, &aGotButtons);
			if ( mChanging || Window()->IsActive() && aGotButtons == 0 ) {
				mAlreadyEntered = true;
				mHot = true;
				Invalidate();
			}
		}
		break;
	case B_EXITED_VIEW:
//		DBEXP("TToolButton::MouseMoved","B_EXITED_VIEW");
		mAlreadyEntered = false;
		mHot = false;
		Invalidate();
		break;
	case B_OUTSIDE_VIEW:
//		DBEXP("TToolButton::MouseMoved","B_OUTSIDE_VIEW");
		break;
	default:
		ASSERT(1, "TToolButton::MouseMoved - other");
	}
}
//------------------------------------------------------------------------------
void	TToolButton::MouseUp(BPoint oPoint)
{
	if ( IsHidden() )
		return;
//	DBEXP("TToolButton::MouseUp","");
	if ( !mChanging )
		return;
	// ボタン内で押されたマウスが放された
	mChanging = false;
	if ( !Bounds().Contains(oPoint) )	// ボタン外で放された
		return;
	// ボタン内で放された
	if ( Behavior() == B_TWO_STATE_BUTTON ) {
		if ( IsRadioMode() && Value() == B_CONTROL_ON ) {
			// ラジオモードで既に押下状態の場合は何もしない
		} else {
			mToolTip->Hide();
			SetValue(!Value());
			Invoke();
		}
	} else {
		mToolTip->Hide();
		Invalidate();
		Invoke();
	}
}
//------------------------------------------------------------------------------
void	TToolButton::ResizeToPreferred()
{
	float		aImageHeight = 0.0;
	font_height	aFontHeight;

	if ( mImages[TB_STATE_ENABLED_OFF] != NULL ) {
		aImageHeight = mImages[TB_STATE_ENABLED_OFF]->Bounds().bottom;
	}
	GetFontHeight(&aFontHeight);
	ResizeTo(MinWidth(), max_c(aImageHeight, aFontHeight.ascent+aFontHeight.descent)+MARGIN_TOOL_BUTTON*2);
	CalcDrawPoints();
}
//------------------------------------------------------------------------------
void	TToolButton::SetAlignment(alignment oForImage, alignment oForLabel)
{
	mImageAlign = oForImage;
	mLabelAlign = oForLabel;
	CalcDrawPoints();
}
//------------------------------------------------------------------------------
status_t	TToolButton::SetBehavior(uint32 oBehavior)
{
	if ( oBehavior == Behavior() )
		return B_OK;
	// 変更
	mBehavior = oBehavior;
	if ( Behavior() == B_ONE_STATE_BUTTON ) {
		SetValue(B_CONTROL_OFF);	// 押されている状態かもしれないのであげる
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TToolButton::SetDefaultImages()
{
	status_t	aResult = B_ERROR;

	if ( mImages[TB_STATE_ENABLED_OFF] == NULL )
		return aResult;
	// EnabledOff → DisabledOff, HotOff
	if ( mImages[TB_STATE_DISABLED_OFF] == NULL ) {
		ConvertFilledCopy(mImages[TB_STATE_ENABLED_OFF],&mImages[TB_STATE_DISABLED_OFF],COLOR_DISABLED_FILLED);
		aResult = B_OK;
	}
	if ( mImages[TB_STATE_HOT_OFF] == NULL ) {
		ConvertTransparentCopy(mImages[TB_STATE_ENABLED_OFF],&mImages[TB_STATE_HOT_OFF],COLOR_HOT_OFF_BG);
		aResult = B_OK;
	}
	// EnabledOff → EnabledOn
	if ( mImages[TB_STATE_ENABLED_ON] == NULL ) {
		ConvertTransparentCopy(mImages[TB_STATE_ENABLED_OFF],&mImages[TB_STATE_ENABLED_ON],COLOR_ENABLED_DISABLED_ON_BG);
		aResult = B_OK;
	}
	// DisabledOff → DisabledOn
	if ( mImages[TB_STATE_DISABLED_ON] == NULL ) {
		ConvertTransparentCopy(mImages[TB_STATE_DISABLED_OFF],&mImages[TB_STATE_DISABLED_ON],COLOR_ENABLED_DISABLED_ON_BG);
		aResult = B_OK;
	}
	return aResult;
}
//------------------------------------------------------------------------------
void	TToolButton::SetDivider(float oDivider)
{
	mDivider = oDivider;
	CalcDrawPoints();
}
//------------------------------------------------------------------------------
status_t	TToolButton::SetImage(int32 oWhich, BBitmap* oImg)
{
	if ( oWhich < 0 || oWhich >= TB_STATE_HOT_ON )
		return B_ERROR;
	if ( mImages[TB_STATE_ENABLED_OFF] == NULL && oWhich != TB_STATE_ENABLED_OFF )
		return B_ERROR;
	if ( oWhich != TB_STATE_ENABLED_OFF && mImages[TB_STATE_ENABLED_OFF]->Bounds() != oImg->Bounds() )
		return B_ERROR;
	delete mImages[oWhich];
	mImages[oWhich] = oImg;
	if ( oWhich == TB_STATE_ENABLED_OFF ) {
		AdjustDivider();
		ResizeToPreferred();
	}
	Invalidate();
	return B_OK;
}
//------------------------------------------------------------------------------
void	TToolButton::SetLabel(const char* oLabel)
{
	SUPER::SetLabel(oLabel);
	ResizeToPreferred();
}
//------------------------------------------------------------------------------
status_t	TToolButton::SetRadioMode(bool oFlag)
{
	vector<TToolButton*>	aBrothers;
	vector<TToolButton*>::iterator	p;

	// 無駄省き＆兄弟の同名関数コールによる無限連鎖防止
	if ( oFlag == IsRadioMode() )
		return B_OK;
	// ラジオモードにするには、TWO STATE でなければならない
	if ( oFlag && Behavior() == B_ONE_STATE_BUTTON )
		return B_ERROR;
	// まず自分を変更
	mRadioMode = oFlag;
	if ( IsRadioMode() ) {
		SetValue(B_CONTROL_OFF);	// 2 つ以上が押されているとまずいので、全部あげてしまう
	}
	// 兄弟を変更
	GetBrothers(&aBrothers);
	p = aBrothers.begin();
	while ( p != aBrothers.end() ) {
		(*p)->SetRadioMode(IsRadioMode());
		p++;
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TToolButton::SetTip(const string& oTipStr)
{
	mToolTip->SetTip(oTipStr);
	return B_OK;
}
//------------------------------------------------------------------------------
void	TToolButton::SetValue(int32 oValue)
{
	vector<TToolButton*>	aBrothers;
	vector<TToolButton*>::iterator	p;

	if ( oValue == Value() )
		return;
	// 自身の設定
	SUPER::SetValue(oValue != B_CONTROL_OFF);
	// 兄弟の設定
	if ( Value() == B_CONTROL_ON && IsRadioMode() ) {
		GetBrothers(&aBrothers);
		p = aBrothers.begin();
		while ( p != aBrothers.end() ) {
			(*p)->SetValue(B_CONTROL_OFF);
			p++;
		}
	}
	Invalidate();
}
//------------------------------------------------------------------------------
string	TToolButton::Tip() const
{
	return mToolTip->Tip();
}
//------------------------------------------------------------------------------
void	TToolButton::WindowActivated(bool oActive)
{
	uint32	aButtons;
	BPoint	aPoint;

	if ( oActive ) {
		GetMouse(&aPoint,&aButtons);
		if ( Bounds().Contains(aPoint) ) {
			mHot = true;
			Invalidate();
		}
	} else {
		mHot = false;
		Invalidate();
	}
}
//------------------------------------------------------------------------------
//==============================================================================
TSeparatorButton::TSeparatorButton(float oHeight)
		: SUPER(BRect(0,0,1,1), "" , NULL, NULL)
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	ResizeTo(TB_SEPARATOR_WIDTH,oHeight);
}
//------------------------------------------------------------------------------
TSeparatorButton::~TSeparatorButton()
{
}
//------------------------------------------------------------------------------
void	TSeparatorButton::AttachedToWindow()
{
	// SUPER の AttachedToWindow() が呼びだされないようにするためのダミー
}
//------------------------------------------------------------------------------
void	TSeparatorButton::Draw(BRect oUpdateRect)
{
	float	aHeight = Bounds().bottom;

	SetHighColor(COLOR_DARK_BORDER);
	StrokeLine(BPoint(TB_SEPARATOR_WIDTH/2,0),BPoint(TB_SEPARATOR_WIDTH/2,aHeight));
	SetHighColor(COLOR_BRIGHT_BORDER);
	StrokeLine(BPoint(TB_SEPARATOR_WIDTH/2+1,0),BPoint(TB_SEPARATOR_WIDTH/2+1,aHeight));
}
//==============================================================================
